@@ -3,12 +3,14 @@
 from functools import partial
 import os
 import re
+from typing import Dict
 
 from debian.copyright import License, NotMachineReadableError
 
 from lintian_brush.copyright import CopyrightUpdater
+from lintian_brush.fixer import report_result
 
-SYNOPSIS_ALIAS = {}
+SYNOPSIS_ALIAS: Dict[str, str] = {}
 updated = set()
 tags = set()
 
@@ -35,12 +37,15 @@ try:
             license = para.license
             if not license or not license.text:
                 continue
-            para.license = License(license.synopsis, re.sub(
+            changed_text = re.sub(
                 '/usr/share/common-licenses/([A-Za-z0-9-.]+)',
-                partial(replace_symlink_path, license.synopsis), license.text))
+                partial(replace_symlink_path, license.synopsis), license.text)
+            if changed_text != license.text:
+                para.license = License(license.synopsis, changed_text)
 except (FileNotFoundError, NotMachineReadableError):
     pass
 
 
-print('Refer to specific version of license %s.' % ', '.join(sorted(updated)))
-print('Fixed-Lintian-Tags: ' + ', '.join(sorted(tags)))
+report_result(
+    'Refer to specific version of license %s.' % ', '.join(sorted(updated)),
+    fixed_lintian_tags=tags)
