@@ -1,9 +1,12 @@
 #!/usr/bin/python3
-import os
 import sys
 from lintian_brush.control import (
     ensure_minimum_debhelper_version,
-    update_control,
+    ControlUpdater,
+    )
+from lintian_brush.fixer import (
+    current_package_version,
+    report_result,
     )
 from lintian_brush.rules import (
     check_cdbs,
@@ -13,36 +16,31 @@ from lintian_brush.rules import (
 minimum_version = "9.20160114"
 
 
-def bump_debhelper(control):
-    control["Build-Depends"] = ensure_minimum_debhelper_version(
-        control.get("Build-Depends", ""), minimum_version)
-
-
 dbg_packages = set()
 dbg_migration_done = set()
 
 
-def del_dbg(control):
-    # Delete the freeradius-dbg package from debian/control
-    package = control["Package"]
-    if package.endswith('-dbg'):
-        if package.startswith('python'):
-            # -dbgsym packages don't include _d.so files for the python
-            # interpreter
-            return
-        dbg_packages.add(control["Package"])
-        control.clear()
+with ControlUpdater() as updater:
+    for control in updater.binaries:
+        # Delete the freeradius-dbg package from debian/control
+        package = control["Package"]
+        if package.endswith('-dbg'):
+            if package.startswith('python'):
+                # -dbgsym packages don't include _d.so files for the python
+                # interpreter
+                continue
+            dbg_packages.add(control["Package"])
+            control.clear()
+
+    if not dbg_packages:
+        # no debug packages found to remove
+        sys.exit(0)
+
+    updater.source["Build-Depends"] = ensure_minimum_debhelper_version(
+        updater.source.get("Build-Depends", ""), minimum_version)
 
 
-update_control(binary_package_cb=del_dbg)
-if not dbg_packages:
-    # no debug packages found to remove
-    sys.exit(0)
-
-update_control(source_package_cb=bump_debhelper)
-
-
-current_version = os.environ["CURRENT_VERSION"]
+current_version = str(current_package_version())
 migrate_version = "<< %s%s" % (
         current_version,
         '' if current_version.endswith('~') else '~')
@@ -78,6 +76,7 @@ if difference:
         sys.exit(2)
     raise Exception("packages missing %r" % difference)
 
-print("Transition to automatic debug package%s (from: %s)." %
-      (("s" if len(dbg_packages) > 1 else ""), ', '.join(dbg_packages)))
-print("Fixed-Lintian-Tags: debian-control-has-obsolete-dbg-package")
+report_result(
+    "Transition to automatic debug package%s (from: %s)." %
+    (("s" if len(dbg_packages) > 1 else ""), ', '.join(dbg_packages)),
+    fixed_lintian_tags=['debian-control-has-obsolete-dbg-package'])
