@@ -3,13 +3,19 @@
 # TODO(jelmer): Read python3 setup.py dist_info
 # TODO(jelmer): Check XS-Go-Import-Path
 
-from debian.changelog import Version
 import os
 import sys
 from lintian_brush import (
-    certainty_sufficient,
     min_certainty,
     )
+from lintian_brush.fixer import (
+    current_package_version,
+    package_is_native,
+    meets_minimum_certainty,
+    net_access_allowed,
+    report_result,
+    trust_package,
+    )
 from lintian_brush.upstream_metadata import (
     UpstreamDatum,
     check_upstream_metadata,
@@ -20,31 +26,37 @@ from lintian_brush.upstream_metadata import (
     filter_bad_guesses,
     ADDON_ONLY_FIELDS,
     upstream_metadata_sort_key,
+    upstream_version,
+    )
+from lintian_brush.yaml import (
+    YamlUpdater,
+    update_ordered_dict,
     )
-from lintian_brush.yaml import YamlUpdater
 
 
-current_version = Version(os.environ['CURRENT_VERSION'])
+fixed_tags = []
 
 
-if not current_version.debian_revision:
+if package_is_native():
     # Native package
     sys.exit(0)
 
 
+current_version = current_package_version()
+
+
 with YamlUpdater('debian/upstream/metadata') as code:
     upstream_metadata = {
         k: UpstreamDatum(k, v, 'certain') for (k, v) in code.items()}
 
     minimum_certainty = os.environ.get('MINIMUM_CERTAINTY')
-    trust_package = os.environ.get('TRUST_PACKAGE') == 'true'
-    net_access = os.environ.get('NET_ACCESS', 'disallow') == 'allow'
+    net_access = net_access_allowed()
 
     # Do some guessing based on what's in the package
     update_from_guesses(
         upstream_metadata, filter_bad_guesses(
             guess_upstream_metadata_items(
-                '.', trust_package=trust_package)))
+                '.', trust_package=trust_package())))
 
     # Then extend that by contacting e.g. SourceForge
     extend_upstream_metadata(
@@ -60,8 +72,8 @@ with YamlUpdater('debian/upstream/metadata') as code:
     if net_access:
         # Verify that online resources actually exist and adjust certainty
         # accordingly.
-        check_upstream_metadata(
-            upstream_metadata, version=current_version.upstream_version)
+        upstream_version = upstream_version(current_version)
+        check_upstream_metadata(upstream_metadata, version=upstream_version)
 
     # Homepage is set in debian/control, so don't add it to
     # debian/upstream/metadata.
@@ -72,7 +84,7 @@ with YamlUpdater('debian/upstream/metadata') as code:
     # debian/upstream/metadata file.
     if 'Name' in upstream_metadata or 'Contact' in upstream_metadata:
         from lintian_brush.copyright import upstream_fields_in_copyright
-        external_present_fields.update(upstream_fields_in_copyright())
+        external_present_fields.update(upstream_fields_in_copyright().keys())
 
     for key, datum in list(upstream_metadata.items()):
         # Drop keys that don't need to be in debian/upsteam/metadata
@@ -80,7 +92,7 @@ with YamlUpdater('debian/upstream/metadata') as code:
             del upstream_metadata[key]
 
         # Drop everything that is below our minimum certainty
-        elif not certainty_sufficient(datum.certainty, minimum_certainty):
+        elif not meets_minimum_certainty(datum.certainty):
             del upstream_metadata[key]
 
     achieved_certainty = min_certainty(
@@ -96,10 +108,25 @@ with YamlUpdater('debian/upstream/metadata') as code:
     if not changed:
         sys.exit(0)
 
-    fixed_tag = not os.path.exists('debian/upstream/metadata')
+    if (('Repository' in changed and 'Repository' not in code) or
+            ('Repository-Browse' in changed and
+                'Repository-Browse' not in code)):
+        fixed_tags.append('upstream-metadata-missing-repository')
 
-    for k, v in sorted(changed.items(), key=upstream_metadata_sort_key):
-        code[k] = v.value
+    if (('Bug-Database' in changed and 'Bug-Database' not in code) or
+            ('Bug-Submit' in changed and 'But-Submit' not in code)):
+        fixed_tags.append('upstream-metadata-missing-bug-tracking')
+
+    # A change that just says the "Name" field is a bit silly
+    if set(changed.keys()) - set(ADDON_ONLY_FIELDS) == set(['Name']):
+        sys.exit(0)
+
+    if not os.path.exists('debian/upstream/metadata'):
+        fixed_tags.append('upstream-metadata-file-is-missing')
+
+    update_ordered_dict(
+        code, [(k, v.value) for (k, v) in changed.items()],
+        key=upstream_metadata_sort_key)
 
     # If there are only add-on-only fields, then just remove the file.
     if not (set(code.keys()) - set(ADDON_ONLY_FIELDS)):
@@ -114,7 +141,7 @@ fields = [
     for k, v in sorted(changed.items())
     ]
 
-print('Set upstream metadata fields: %s.' % ', '.join(sorted(fields)))
-print('Certainty: %s' % achieved_certainty)
-if fixed_tag:
-    print('Fixed-Lintian-Tags: upstream-metadata-file-is-missing')
+report_result(
+    'Set upstream metadata fields: %s.' % ', '.join(sorted(fields)),
+    fixed_lintian_tags=fixed_tags,
+    certainty=achieved_certainty)
