@@ -17,7 +17,7 @@
 
 """Automatically fix lintian issues."""
 
-from debian.changelog import Changelog
+from debian.changelog import Changelog, Version
 import errno
 import io
 import os
@@ -28,6 +28,7 @@ import sys
 import tempfile
 import time
 import traceback
+from typing import Optional, List, Sequence, Iterator, Iterable
 import warnings
 
 from breezy import ui
@@ -43,6 +44,7 @@ from breezy.osutils import is_inside
 from breezy.rename_map import RenameMap
 from breezy.trace import note, warning
 from breezy.transform import revert
+from breezy.workingtree import WorkingTree
 
 from debian.deb822 import Deb822
 
@@ -50,7 +52,7 @@ from debian.deb822 import Deb822
 from .reformatting import FormattingUnpreservable
 
 
-__version__ = (0, 58)
+__version__ = (0, 65)
 version_string = '.'.join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ['certain', 'confident', 'likely', 'possible', None]
 DEFAULT_MINIMUM_CERTAINTY = 'certain'
@@ -62,6 +64,19 @@ DEFAULT_URLLIB_TIMEOUT = 3
 class NoChanges(Exception):
     """Script didn't make any changes."""
 
+    def __init__(self, fixer, comment=None):
+        super(NoChanges, self).__init__(fixer, comment)
+        self.fixer = fixer
+
+
+class NotCertainEnough(NoChanges):
+    """Script made changes but with too low certainty."""
+
+    def __init__(self, fixer, certainty, minimum_certainty):
+        super(NotCertainEnough, self).__init__(fixer)
+        self.certainty = certainty
+        self.minimum_certainty = minimum_certainty
+
 
 class FixerFailed(Exception):
     """Base class for fixer script failures."""
@@ -101,6 +116,10 @@ class FixerScriptFailed(FixerFailed):
 class DescriptionMissing(Exception):
     """The fixer script did not provide a description on stdout."""
 
+    def __init__(self, fixer):
+        super(DescriptionMissing, self).__init__(fixer)
+        self.fixer = fixer
+
 
 class NotDebianPackage(Exception):
     """The specified directory does not contain a Debian package."""
@@ -152,7 +171,7 @@ class Fixer(object):
     fixer addresses.
     """
 
-    def __init__(self, name, lintian_tags=None):
+    def __init__(self, name: str, lintian_tags: List[str] = None):
         self.name = name
         self.lintian_tags = lintian_tags or []
 
@@ -193,7 +212,7 @@ def parse_script_fixer_output(text):
             lines.append(line)
         else:
             if key == 'Fixed-Lintian-Tags':
-                fixed_tags = value.strip().split(',')
+                fixed_tags = [tag.strip() for tag in value.strip().split(',')]
             elif key == 'Certainty':
                 certainty = value.strip()
             elif key == 'Patch-Name':
@@ -288,7 +307,7 @@ class PythonScriptFixer(Fixer):
             os.chdir(old_cwd)
 
         if retcode == 2:
-            raise NoChanges()
+            raise NoChanges(self)
         if retcode != 0:
             raise FixerScriptFailed(self.script_path, retcode, err)
 
@@ -298,17 +317,24 @@ class PythonScriptFixer(Fixer):
 class ScriptFixer(Fixer):
     """A fixer that is implemented as a shell/python/etc script."""
 
-    def __init__(self, name, lintian_tags, script_path):
+    def __init__(self, name: str, lintian_tags: List[str], script_path: str):
         super(ScriptFixer, self).__init__(name, lintian_tags)
         self.script_path = script_path
 
     def __repr__(self):
         return "<ScriptFixer(%r)>" % self.name
 
-    def run(self, basedir, package, current_version, compat_release,
-            minimum_certainty=DEFAULT_MINIMUM_CERTAINTY,
-            trust_package=False, allow_reformatting=False,
-            net_access=True, opinionated=False, diligence=0):
+    def run(self,
+            basedir: str,
+            package: str,
+            current_version: Version,
+            compat_release: str,
+            minimum_certainty: str = DEFAULT_MINIMUM_CERTAINTY,
+            trust_package: bool = False,
+            allow_reformatting: bool = False,
+            net_access: bool = True,
+            opinionated: bool = False,
+            diligence: int = 0):
         env = determine_env(
             package=package,
             current_version=current_version,
@@ -323,9 +349,9 @@ class ScriptFixer(Fixer):
             p = subprocess.Popen(self.script_path, cwd=basedir,
                                  stdout=subprocess.PIPE, stderr=stderr,
                                  env=env)
-            (description, err) = p.communicate("")
+            (description, err) = p.communicate(b"")
             if p.returncode == 2:
-                raise NoChanges()
+                raise NoChanges(self)
             if p.returncode != 0:
                 stderr.seek(0)
                 raise FixerScriptFailed(
@@ -334,7 +360,7 @@ class ScriptFixer(Fixer):
         return parse_script_fixer_output(description.decode('utf-8'))
 
 
-def find_fixers_dir():
+def find_fixers_dir() -> str:
     """Find the local directory with lintian fixer scripts."""
     local_dir = os.path.abspath(os.path.join(
         os.path.dirname(__file__), '..', 'fixers'))
@@ -349,7 +375,7 @@ def find_fixers_dir():
     return '/usr/share/lintian-brush/fixers'
 
 
-def read_desc_file(path):
+def read_desc_file(path: str) -> Iterator[Fixer]:
     """Read a description file.
 
     Args:
@@ -373,31 +399,34 @@ def read_desc_file(path):
                 yield ScriptFixer(name, tags, script_path)
 
 
-def select_fixers(fixers, names, exclude=None):
+def select_fixers(fixers: List[Fixer],
+                  names: List[str],
+                  exclude: Optional[Iterable[str]] = None) -> List[Fixer]:
     """Select fixers by name, from a list.
 
     Args:
       fixers: List of Fixer objects
-      names: List of names to select
-      exclude: List of names to exclude
+      names: Set of names to select
+      exclude: Set of names to exclude
     Raises:
       KeyError: if one of the names did not exist
     """
-    names = set(names)
+    names_set = set(names)
     if exclude:
         for name in exclude:
-            if name not in names:
+            if name not in names_set:
                 raise KeyError(name)
-            names.remove(name)
+            names_set.remove(name)
     available = set([f.name for f in fixers])
-    missing = names - available
+    missing = names_set - available
     if missing:
         raise KeyError(missing.pop())
     # Preserve order
-    return [f for f in fixers if f.name in names]
+    return [f for f in fixers if f.name in names_set]
 
 
-def available_lintian_fixers(fixers_dir=None):
+def available_lintian_fixers(
+        fixers_dir: Optional[str] = None) -> Iterator[Fixer]:
     """Return a list of available lintian fixers.
 
     Args:
@@ -414,7 +443,7 @@ def available_lintian_fixers(fixers_dir=None):
             yield fixer
 
 
-def increment_version(v):
+def increment_version(v: Version) -> None:
     """Increment a version number.
 
     For native packages, increment the main version number.
@@ -431,7 +460,7 @@ def increment_version(v):
                 '\\d+$', lambda x: str(int(x.group())+1), v.upstream_version)
 
 
-def delete_items(deletables, dry_run=False):
+def delete_items(deletables, dry_run: bool = False):
     """Delete files in the deletables iterable"""
     def onerror(function, path, excinfo):
         """Show warning for errors seen by rmtree.
@@ -455,7 +484,7 @@ def delete_items(deletables, dry_run=False):
                     'unable to remove "{0}": {1}.'.format(path, e.strerror))
 
 
-def get_committer(tree):
+def get_committer(tree: WorkingTree) -> str:
     """Get the committer string for a tree.
 
     Args:
@@ -470,18 +499,14 @@ def get_committer(tree):
         email = os.environ.get("GIT_COMMITTER_EMAIL")
         if user is None:
             try:
-                user = cs.get(("user", ), "name")
+                user = cs.get(("user", ), "name").decode('utf-8')
             except KeyError:
                 user = None
-            else:
-                user = user.decode('utf-8')
         if email is None:
             try:
-                email = cs.get(("user", ), "email")
+                email = cs.get(("user", ), "email").decode('utf-8')
             except KeyError:
                 email = None
-            else:
-                email = email.decode('utf-8')
         if user and email:
             return user + " <" + email + ">"
         from breezy.config import GlobalStack
@@ -491,7 +516,10 @@ def get_committer(tree):
         return config.get('email')
 
 
-def only_changes_last_changelog_block(tree, changelog_path, changes):
+def only_changes_last_changelog_block(
+        tree: WorkingTree,
+        changelog_path: str,
+        changes) -> bool:
     """Check whether the only change in a tree is to the last changelog entry.
 
     Args:
@@ -524,7 +552,8 @@ def only_changes_last_changelog_block(tree, changelog_path, changes):
         return str(new_cl) == str(old_cl)
 
 
-def reset_tree(local_tree, dirty_tracker=None, subpath='.'):
+def reset_tree(local_tree: WorkingTree, dirty_tracker=None,
+               subpath: str = '.') -> None:
     """Reset a tree back to its basis tree.
 
     This will leave ignored and detritus files alone.
@@ -532,6 +561,7 @@ def reset_tree(local_tree, dirty_tracker=None, subpath='.'):
     Args:
       local_tree: tree to work on
       dirty_tracker: Optional dirty tracker
+      subpath: Subpath to operate on
     """
     if dirty_tracker and not dirty_tracker.is_dirty():
         return
@@ -542,7 +572,8 @@ def reset_tree(local_tree, dirty_tracker=None, subpath='.'):
     delete_items(deletables)
 
 
-def certainty_sufficient(actual_certainty, minimum_certainty):
+def certainty_sufficient(actual_certainty: str,
+                         minimum_certainty: Optional[str]) -> bool:
     """Check if the actual certainty is sufficient.
 
     Args:
@@ -562,7 +593,7 @@ def certainty_sufficient(actual_certainty, minimum_certainty):
     return actual_confidence <= minimum_confidence
 
 
-def check_clean_tree(local_tree):
+def check_clean_tree(local_tree: WorkingTree) -> None:
     """Check that a tree is clean and has no pending changes or unknown files.
 
     Args:
@@ -577,13 +608,17 @@ def check_clean_tree(local_tree):
         raise PendingChanges(local_tree)
 
 
-def add_changelog_entry(tree, path, summary, qa=False):
+def add_changelog_entry(
+        tree: WorkingTree, path: str, summary: str,
+        qa: bool = False, suppress_warnings: bool = False) -> None:
     """Add a changelog entry.
 
     Args:
       tree: Tree to edit
       path: Path to the changelog file
       summary: Entry to add
+      qa: Whether to add a qa entry
+      suppress_warnings: Whether to suppress any warnings from 'dch'
     """
     args = ["dch", "--no-auto-nmu"]
     if qa:
@@ -596,7 +631,7 @@ def add_changelog_entry(tree, path, summary, qa=False):
     (stdout, stderr) = p.communicate(b"\n")
     if stdout:
         raise ValueError(stdout)
-    if stderr:
+    if stderr and not suppress_warnings:
         end = b'dch: Did you see that warning?  Press RETURN to continue...\n'
         if stderr.endswith(end):
             stderr = stderr[:-len(end)]
@@ -604,69 +639,6 @@ def add_changelog_entry(tree, path, summary, qa=False):
             warning('%s', line)
 
 
-def gbp_conf_has_dch_section(tree, path=''):
-    try:
-        gbp_conf_path = os.path.join(path, 'debian/gbp.conf')
-        gbp_conf_text = tree.get_file_text(gbp_conf_path)
-    except NoSuchFile:
-        return False
-    try:
-        import configparser
-    except ImportError:
-        return None
-    else:
-        parser = configparser.ConfigParser(defaults={}, strict=False)
-        parser.read_string(
-            gbp_conf_text.decode('utf-8', errors='replace'), gbp_conf_path)
-        return parser.has_section('dch')
-
-
-def all_sha_prefixed(cl):
-    sha_prefixed = 0
-    for change in cl.changes():
-        if not change.startswith('  * '):
-            continue
-        if re.match(r'  \* \[[0-9a-f]{7}\] ', change):
-            sha_prefixed += 1
-        else:
-            return False
-    return (sha_prefixed > 0)
-
-
-_changelog_policy_noted = False
-
-
-def guess_update_changelog(tree, path='', cl=None):
-    """Guess whether the changelog should be updated.
-
-    Args:
-      tree: Tree to edit
-      path: Path to packaging in tree
-    """
-    global _changelog_policy_noted
-    if gbp_conf_has_dch_section(tree, path):
-        if not _changelog_policy_noted:
-            note('Assuming changelog does not need to be updated, '
-                 'since there is a [dch] section in gbp.conf.')
-            _changelog_policy_noted = True
-        return False
-
-    # TODO(jelmes): Do something more clever here, perhaps looking at history
-    # of the changelog file?
-    changelog_path = os.path.join(path, 'debian/changelog')
-    if cl is None:
-        try:
-            with tree.get_file(changelog_path) as f:
-                cl = Changelog(f, max_blocks=1)
-        except NoSuchFile:
-            cl = None
-
-    if cl and all_sha_prefixed(cl[0]):
-        return False
-
-    return True
-
-
 def has_non_debian_changes(changes):
     for change in changes:
         try:
@@ -679,12 +651,19 @@ def has_non_debian_changes(changes):
     return False
 
 
-def run_lintian_fixer(local_tree, fixer, committer=None,
-                      update_changelog=None, compat_release=None,
-                      minimum_certainty=None, trust_package=False,
-                      allow_reformatting=False, dirty_tracker=None,
-                      subpath='.', net_access=True, opinionated=None,
-                      diligence=0):
+def run_lintian_fixer(local_tree: WorkingTree,
+                      fixer: Fixer,
+                      committer: Optional[str] = None,
+                      update_changelog: Optional[bool] = None,
+                      compat_release: Optional[str] = None,
+                      minimum_certainty: Optional[str] = None,
+                      trust_package: bool = False,
+                      allow_reformatting: bool = False,
+                      dirty_tracker=None,
+                      subpath: str = '.',
+                      net_access: bool = True,
+                      opinionated: Optional[bool] = None,
+                      diligence: int = 0):
     """Run a lintian fixer on a tree.
 
     Args:
@@ -744,7 +723,8 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         raise
     if not certainty_sufficient(result.certainty, minimum_certainty):
         reset_tree(local_tree, dirty_tracker, subpath)
-        raise NoChanges("Certainty of script's changes not high enough")
+        raise NotCertainEnough(fixer, result.certainty, minimum_certainty)
+    specific_files: Optional[List[str]]
     if dirty_tracker:
         relpaths = dirty_tracker.relpaths()
         # Sort paths so that directories get added before the files they
@@ -757,7 +737,7 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
             p for p in relpaths
             if local_tree.is_versioned(p)]
         if not specific_files:
-            raise NoChanges("Script didn't make any changes")
+            raise NoChanges(fixer, "Script didn't make any changes")
     else:
         local_tree.smart_add([local_tree.abspath(subpath)])
         specific_files = [subpath] if subpath != '.' else None
@@ -772,10 +752,10 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         want_unversioned=False, require_versioned=True))
 
     if len(local_tree.get_parent_ids()) <= 1 and not changes:
-        raise NoChanges("Script didn't make any changes")
+        raise NoChanges(fixer, "Script didn't make any changes")
 
     if not result.description:
-        raise DescriptionMissing()
+        raise DescriptionMissing(fixer)
 
     summary = result.description.splitlines()[0]
 
@@ -797,6 +777,7 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         summary = 'Add patch %s: %s' % (patch_name, summary)
 
     if update_changelog is None:
+        from .detect_gbp_dch import guess_update_changelog
         update_changelog = guess_update_changelog(local_tree, subpath, cl)
 
     if update_changelog and only_changes_last_changelog_block(
@@ -845,7 +826,9 @@ class ManyResult(object):
         return iter(self.__tuple__())
 
 
-def get_dirty_tracker(local_tree, subpath='', use_inotify=None):
+def get_dirty_tracker(local_tree: WorkingTree,
+                      subpath: str = '',
+                      use_inotify: bool = None):
     """Create a dirty tracker object."""
     if use_inotify is True:
         from .dirty_tracker import DirtyTracker
@@ -861,12 +844,20 @@ def get_dirty_tracker(local_tree, subpath='', use_inotify=None):
             return DirtyTracker(local_tree, subpath)
 
 
-def run_lintian_fixers(local_tree, fixers, update_changelog=True,
-                       verbose=False, committer=None,
-                       compat_release=None, minimum_certainty=None,
-                       trust_package=False, allow_reformatting=False,
-                       use_inotify=None, subpath='.', net_access=True,
-                       opinionated=None, diligence=0):
+def run_lintian_fixers(local_tree: WorkingTree,
+                       fixers: List[Fixer],
+                       update_changelog: bool = True,
+                       verbose: bool = False,
+                       committer: Optional[str] = None,
+                       compat_release: Optional[str] = None,
+                       minimum_certainty: Optional[str] = None,
+                       trust_package: bool = False,
+                       allow_reformatting: bool = False,
+                       use_inotify: Optional[bool] = None,
+                       subpath: str = '.',
+                       net_access: bool = True,
+                       opinionated: Optional[bool] = None,
+                       diligence: int = 0):
     """Run a set of lintian fixers on a tree.
 
     Args:
@@ -928,6 +919,12 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
                 if verbose:
                     note('Fixer %r failed to run.', fixer.name)
                     sys.stderr.write(str(e))
+            except NotCertainEnough as e:
+                if verbose:
+                    note('Fixer %r made changes but not high enough '
+                         'certainty (was %r, needed %r). (took: %.2fs)',
+                         fixer.name, e.certainty, e.minimum_certainty,
+                         time.time() - start)
             except NoChanges:
                 if verbose:
                     note('Fixer %r made no changes. (took: %.2fs)',
@@ -940,22 +937,22 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
     return ret
 
 
-def certainty_to_confidence(certainty):
+def certainty_to_confidence(certainty: Optional[str]) -> Optional[int]:
     if certainty in ('unknown', None):
         return None
     return SUPPORTED_CERTAINTIES.index(certainty)
 
 
-def confidence_to_certainty(confidence):
+def confidence_to_certainty(confidence: Optional[int]) -> str:
     if confidence is None:
         return 'unknown'
     try:
-        return SUPPORTED_CERTAINTIES[confidence]
+        return SUPPORTED_CERTAINTIES[confidence] or 'unknown'
     except IndexError:
         raise ValueError(confidence)
 
 
-def min_certainty(certainties):
+def min_certainty(certainties: Sequence[str]) -> str:
     return confidence_to_certainty(
         max([certainty_to_confidence(c)
             for c in certainties] + [0]))
