@@ -39,7 +39,7 @@ from iniparse.ini import (
     readline_iterator,
     )
 
-from .reformatting import edit_formatted_file
+from .reformatting import Updater
 
 import os
 
@@ -92,8 +92,8 @@ class Section(ConfigNamespace):
                         values.remove(v)
                 option.value = ' '.join(values)
                 if not option.value:
-                    for l in self._lines:
-                        l.contents.remove(option)
+                    for line in self._lines:
+                        line.contents.remove(option)
             if remaining:
                 obj = LineContainer(
                     OptionLine(key, ' '.join(remaining), separator='='))
@@ -110,21 +110,21 @@ class Section(ConfigNamespace):
             self._options[key].value = value
 
     def __delitem__(self, key):
-        for l in self._lines:
+        for line in self._lines:
             remaining = []
-            for o in l.contents:
+            for o in line.contents:
                 if isinstance(o, LineContainer):
                     if key != o.name:
                         remaining.append(o)
                 else:
                     remaining.append(o)
-            l.contents = remaining
+            line.contents = remaining
         del self._options[key]
 
     def __iter__(self):
         d = set()
-        for l in self._lines:
-            for x in l.contents:
+        for line in self._lines:
+            for x in line.contents:
                 if isinstance(x, LineContainer):
                     ans = x.name
                     if ans not in d:
@@ -152,8 +152,8 @@ class OptionList(object):
             if k >= i and k < i + len(vals):
                 del vals[k - i]
                 o.value = ' '.join(vals)
-                for l in self._section._lines:
-                    l.contents.remove(o)
+                for line in self._section._lines:
+                    line.contents.remove(o)
                 break
             i += len(vals)
         else:
@@ -355,29 +355,17 @@ def systemd_service_files(path='debian', exclude_links=True):
         yield subpath
 
 
-class SystemdServiceUpdater(object):
+class SystemdServiceUpdater(Updater):
 
-    def __init__(self, path):
-        self.path = path
+    def _parse(self, content):
+        return UnitFile(StringIO(self._orig_content))
 
-    def __enter__(self):
-        with open(self.path, 'r') as f:
-            self._orig_content = f.read()
+    def _format(self, parsed):
+        return str(parsed)
 
-        self.file = UnitFile(StringIO(self._orig_content))
-        self._rewritten_content = self.dump()
-        return self
-
-    def dump(self):
-        return str(self.file)
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        updated_content = self.dump()
-
-        self.changed = edit_formatted_file(
-            self.path, self._orig_content, self._rewritten_content,
-            updated_content)
-        return False
+    @property
+    def file(self):
+        return self._parsed
 
 
 def update_service_file(path, cb):
