@@ -53,7 +53,7 @@ class FixerTestCase(unittest.TestCase):
                         symlinks=True, ignore=ignore)
 
     def id(self):
-        return "%s.%s" % (__name__, self._fixer_name, self._test_name)
+        return "%s.%s.%s" % (__name__, self._fixer_name, self._test_name)
 
     def __str__(self):
         return 'fixer test: %s for %s' % (self._test_name, self._fixer_name)
@@ -78,11 +78,12 @@ class FixerTestCase(unittest.TestCase):
             current_version = '1.0-1'
         env['CURRENT_VERSION'] = str(current_version)
         env['NET_ACCESS'] = 'disallow'
+        env['MINIMUM_CERTAINTY'] = 'possible'
         env_path = os.path.join(self._path, 'env')
         if os.path.exists(env_path):
             with open(env_path, 'r') as f:
-                for l in f:
-                    key, value = l.rstrip('\n').split('=')
+                for line in f:
+                    key, value = line.rstrip('\n').split('=')
                     env[key] = value
         p = subprocess.Popen(self._fixer.script_path, cwd=self._testdir,
                              stdout=subprocess.PIPE,
