@@ -19,6 +19,7 @@
 
 from debian.copyright import (
     FilesParagraph,
+    LicenseParagraph,
     License,
     )
 
@@ -28,7 +29,7 @@ from breezy.tests import (
 
 from lintian_brush.copyright import (
     NotMachineReadableError,
-    update_copyright,
+    CopyrightUpdater,
     )
 from lintian_brush.reformatting import (
     FormattingUnpreservable,
@@ -48,7 +49,11 @@ Files: *
 License: GPL
 Copyright: 2012...
 """)])
-        self.assertRaises(FormattingUnpreservable, update_copyright, None)
+
+        def dummy():
+            with CopyrightUpdater() as updater:
+                updater.copyright.header.upstream_name = 'llintian-brush'
+        self.assertRaises(FormattingUnpreservable, dummy)
 
     def test_old_style(self):
         self.build_tree_contents([('debian/', ), ('debian/copyright', """\
@@ -56,7 +61,11 @@ This package was debianized in 1995 by Joe Example <joe@example.com>
 
 It was downloaded from ftp://ftp.example.com/pub/blah.
 """)])
-        self.assertRaises(NotMachineReadableError, update_copyright, None)
+
+        def dummy():
+            with CopyrightUpdater():
+                pass
+        self.assertRaises(NotMachineReadableError, dummy)
 
     def test_modify(self):
         self.build_tree_contents([('debian/', ), ('debian/copyright', """\
@@ -69,11 +78,11 @@ License: GPL
 Copyright: 2012...
 """)])
 
-        def add_stanza(copyright):
-            copyright.add_files_paragraph(FilesParagraph.create(
+        with CopyrightUpdater() as updater:
+            updater.copyright.add_files_paragraph(FilesParagraph.create(
                 ['foo.c'], "2012 Joe Example",
                 License("Apache")))
-        self.assertTrue(update_copyright(add_stanza))
+        self.assertTrue(updater.changed)
         self.assertFileEqual("""\
 Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
 Upstream-Name: lintian-brush
@@ -87,3 +96,44 @@ Files: foo.c
 Copyright: 2012 Joe Example
 License: Apache
 """, 'debian/copyright')
+
+    def test_add_paragraph(self):
+        self.build_tree_contents([('debian/', ), ('debian/copyright', """\
+Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
+Upstream-Name: lintian-brush
+Upstream-Contact: Jelmer <jelmer@samba.org>
+""")])
+
+        with CopyrightUpdater() as updater:
+            updater.copyright.add_license_paragraph(LicenseParagraph.create(
+                License("Blah", 'Blah\nblah blah\nblah\n\n')))
+        self.assertTrue(updater.changed)
+        self.assertFileEqual("""\
+Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
+Upstream-Name: lintian-brush
+Upstream-Contact: Jelmer <jelmer@samba.org>
+
+License: Blah
+ Blah
+ blah blah
+ blah
+ .
+""", 'debian/copyright')
+
+    def test_preserve_whitespace(self):
+        self.build_tree_contents([('debian/', ), ('debian/copyright', """\
+Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
+Upstream-Name: lintian-brush
+Upstream-Contact: Jelmer <jelmer@samba.org>
+
+License: Blah
+ blah
+ .
+""")])
+
+        with CopyrightUpdater() as updater:
+            license_para = list(updater.copyright.all_license_paragraphs())[0]
+            self.assertEqual(
+                'License: Blah\n blah\n .\n',
+                license_para.dump())
+            self.assertEqual("blah\n", license_para.license.text)
