@@ -20,6 +20,7 @@
 import os
 import re
 import sys
+from typing import Type
 
 from debian.changelog import (
     Changelog,
@@ -46,7 +47,6 @@ from lintian_brush import (
     available_lintian_fixers,
     check_clean_tree,
     add_changelog_entry,
-    guess_update_changelog,
     certainty_sufficient,
     min_certainty,
     certainty_to_confidence,
@@ -650,8 +650,8 @@ class LintianBrushVersion(TestCase):
                 'Running outside of source tree?')
         # TODO(jelmer): Surely there's a better way of doing this?
         with open('setup.py', 'r') as f:
-            for l in f:
-                m = re.match(r'[ ]*version="(.*)",', l)
+            for line in f:
+                m = re.match(r'[ ]*version="(.*)",', line)
                 if m:
                     setup_version = m.group(1)
                     break
@@ -779,7 +779,7 @@ Patch-Name: aname
 
 class BaseScriptFixerTests(object):
 
-    script_fixer_cls = None
+    script_fixer_cls: Type[Fixer]
 
     def create_fixer(self, code):
         self.build_tree_contents([
@@ -906,6 +906,7 @@ lintian-brush (0.35) unstable; urgency=medium
         tree.add(['debian', 'debian/changelog'])
         self.overrideEnv('DEBFULLNAME', 'Jane Example')
         self.overrideEnv('DEBEMAIL', 'jane@example.com')
+        self.overrideEnv('DEBCHANGE_VENDOR', 'debian')
         add_changelog_entry(tree, 'debian/changelog', 'Add a foo')
         text = tree.get_file_text('debian/changelog').decode()
         lines = text.splitlines()
@@ -949,40 +950,3 @@ lintian-brush (0.35) UNRELEASED; urgency=medium
 
  -- Joe Example <joe@example.com>  Fri, 04 Oct 2019 02:36:13 +0000
 """, 'debian/changelog')
-
-
-class GuessUpdateChangelogTests(TestCaseWithTransport):
-
-    def test_no_gbp_conf(self):
-        tree = self.make_branch_and_tree('.')
-        self.assertTrue(guess_update_changelog(tree))
-
-    def test_gbp_conf_dch(self):
-        tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([
-            ('debian/', ),
-            ('debian/gbp.conf', """\
-[dch]
-""")])
-        tree.add(['debian', 'debian/gbp.conf'])
-        self.assertFalse(guess_update_changelog(tree))
-
-    def test_changelog_sha_prefixed(self):
-        tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([
-            ('debian/', ),
-            ('debian/changelog', """\
-blah (0.20.1) unstable; urgency=medium
-
-  [ Somebody ]
-  * [ebb7c31] do a thing
-  * [629746a] do another thing that actually requires us to wrap lines
-    and then
-
-  [ Somebody Else ]
-  * [b02b435] do another thing
-
- -- Joe User <joe@example.com>  Tue, 19 Nov 2019 15:29:47 +0100
-""")])
-        tree.add(['debian', 'debian/changelog'])
-        self.assertFalse(guess_update_changelog(tree))
