@@ -95,6 +95,10 @@ class FixUpGitUrlTests(TestCase):
         self.assertEqual(
             'https://gitlab.freedesktop.org/xorg/xserver',
             fixup_broken_git_url('git://anongit.freedesktop.org/xorg/xserver'))
+        self.assertEqual(
+            'https://gitlab.freedesktop.org/xorg/lib/libSM',
+            fixup_broken_git_url(
+                'git://anongit.freedesktop.org/git/xorg/lib/libSM'))
 
     def test_gnome(self):
         self.assertEqual(
@@ -147,6 +151,14 @@ class DetermineBrowserUrlTests(TestCase):
             'https://github.com/jelmer/dulwich/tree/HEAD/blah',
             determine_browser_url(
                 'git', 'git://github.com/jelmer/dulwich [blah]'))
+        self.assertEqual(
+            'https://git.sv.gnu.org/cgit/rcs.git',
+            determine_browser_url(
+                'git', 'https://git.sv.gnu.org/git/rcs.git'))
+        self.assertEqual(
+            'https://git.savannah.gnu.org/cgit/rcs.git',
+            determine_browser_url(
+                'git', 'git://git.savannah.gnu.org/rcs.git'))
 
 
 class PlausibleUrlTests(TestCase):
